(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     26063,       1038]*)
(*NotebookOutlinePosition[     26982,       1068]*)
(*  CellTagsIndexPosition[     26938,       1064]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["Rubik Implementation Notes", "Title"],

Cell["General", "Subtitle"],

Cell[CellGroupData[{

Cell["Directory name conventions", "Section"],

Cell["\<\
We want the files and directory structure to be as portable as \
possible and independent of Mathematica's installation and AddOn directories. \
This is important for those running over a network, or under unix and without \
priveleges to  add to or alter these directories.\
\>", "Text"],

Cell["\<\
Different systems use different conventions for folders;\"folder\" \
on the Mac, \"Folder\" under  windows, \"Directory\" under unix/linux. We \
have used the suffix \"Folder\" in this directory structure.\
\>", "Text"],

Cell["\<\
Non alphanumeric characters greatly enhance the readability and \
manipulability of files and folders.
File and folder names can contain spaces in all systems-but no other \
aphanumeric characters are always safe. \
\>", "Text"],

Cell["\<\
Capitalization seems the only way to enhance readability of file \
names (subject to context conventions).  It would be nice to at least retain \
spaces in folder and file names. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Contexts names", "Section"],

Cell["\<\
Often it is more convenient to maintain small files containing \
(function and other) definitions that are combined into one context. This is \
so especially during development where the relationship is still fluid. For \
this reason there are \"Fn\" files that simply contain the declarations and \
\"Pk\" or package files that contain the package context statements. They \
also realize the separation of interface (Import/Export in Modula II for \
example, headers in C++) and implementation, and, as in Haskell, allow more \
flexible interfacing.\
\>", "Text"],

Cell["\<\
Since context names cannot contain spaces:
either there is a separate \"spaced\" name for the files and explicit Gets
or spaces are eliminated from directory and file names.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Identifiers", "Section"],

Cell[CellGroupData[{

Cell["hungarian convention", "Subsubsection"],

Cell[" A hungarian convention is used for some names.", "Text"],

Cell["\<\
-\"k\" or c for constants
-\"T\" for type and \"d\" for data (type)
-\"..Of..\" in function names because more often functions are referenced by \
their output type.
 (except when readability favours e.g. \"Rofa\").
-\"a\" or \"an\" or \"the\" for an instance of a type, e.g. aPoint, anAgent.
-\"...To..\" in rules since they use left to right application.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Labelled Permutations", "Subsubsection"],

Cell["\<\
This  is an important structure in these packages and one that is \
worth discussing in the light of general conventions for identifiers and \
types, and also because we violate some of our own conventions with it.\
\>", \
"Text"],

Cell["\<\
We first make a preliminary remark about the data structure chosen \
for permutations.\
\>", "Text"],

Cell["\<\
Permutations are instantiated by rules. This seems more natural, \
closer to the mathematics (functions as relations) and more efficient and \
flexible (can apply permutations to deeply nested structures and compose \
them) than the list structure used in the Combinatoria package.\
\>", "Text"],

Cell["\<\
A fundamental role  in both the group theory and the Rubik's cube \
(packages)  is played by symbolic permutation actions.  \
\>", "Text"],

Cell[TextData[{
  "In a strictly typed language the computation of a composition of \
permutations such as ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Sigma]", "\[SmallCircle]", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"-", "1"}]]}], TraditionalForm]]],
  "is achieved by binding the \[Sigma] to its permuation and computing the \
inverse and composition. Even normal order/lazy evaluation does this. The \
computation is on  the semantics, but at the term or algbebraic level the \
computation is trivial. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " allows us to mix this meta/symbolic level and computation. On paper we \
control which level we are on at each step by hand (and idiosynchratically). \
"
}], "Text"],

Cell["\<\
To do this systematically there are two choices
- operate on the term level, and then instantiate explicitly via a semantic \
mapping of all terms, or
-use objects with both term and function data.\
\>", "Text"],

Cell["\<\
We choose the second for permutations acting on a set. The terms \
are thought of as labels. \
\>", "Text"],

Cell[TextData[{
  "The word \"action\" has so many mathematical applications that we choose \
the more specific word \"Permutation\".\nHence the first choice would be the \
data structure \"TLabelledPermutationRules[",
  StyleBox["formula",
    FontSlant->"Italic"],
  ",",
  StyleBox["permutationrules",
    FontSlant->"Italic"],
  "]\" ."
}], "Text"],

Cell["\<\
This data structure occurs often, both in data files and functions, \
that we have compromised and used \"LPerm\" for this type head.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Morphemes", "Subsection"],

Cell[CellGroupData[{

Cell["comments", "Subsubsection"],

Cell["\<\
The identifiers are constructed from morphemes. The non-obvious \
descriptions of the morphemes used as components of lexemes is as \
follows.\
\>", "Text"],

Cell["\<\
We try to delineate morphemes within lexemes by capitalization. \
This is crucial for readability.\
\>", "Text"],

Cell["\<\
There are conventions that fix the case of some morphemes. We use \
upper case for matrix variables for instance, and lower case for prepositions \
like \"a\".\
\>", "Text"],

Cell["\<\
One could achieve more flexible identifier structure by using \
Notations. Many of these identifiers are used however for internal functions \
(to which notations are mapped) so that only addresses some of the \
limitations.\
\>", "Text"],

Cell["\<\
Some abbreviation is needed to stop identifiers becoming long \
sentences in themselves.\
\>", "Text"],

Cell["\<\
The files were processed with Mathematica functions for analysing \
and transforming words and morphemes.\
\>", "SmallText"]
}, Closed]],

Cell[CellGroupData[{

Cell["component morpheme descriptions:", "Subsubsection"],

Cell["\<\
\"a\" is a prefix, indefinite article e.g. ar ( a rotation label) \
aVector, aCube...\
\>", "Text"],

Cell["Note the dual role of \"q\",\"Q\".", "Text"],

Cell["\<\
3D
90
a -is a prefix, indefinite article e.g. ar ( a rotation label) aVector, \
aCube..
Absolute
Action
Add
After
ak   - a color (label)
All
an  - indefinite article prefix
Animate
Animation
Association
Axis
Back
Basis
Block
Boolean
Boxes
Boxing
Butt - button
Button
By
Cell
Clear
Close
Color
Colored
Command
Config
Configuration
Create
Cube - a colored mini-face configuration describing a rubik cube state.
Curr
Current
Cycle
Data
Define
Display
Do
Dom -domain
Expr- an expression, e.g. for a cell, or  (a more general than mathematical) \
formula.
f    - a face (label)
Face
Facial
Fast
Fill
Fixed
Fn - a function (as parameter or data)
Form -in a formula action type.
Formula
Formulae
g - head of a group element or rubik generator.
Gen - a generator
Get
gop -  a rubik generator operation i.e. g[r,s]
Graphics
Grid
Icon
Index
Indices
Initial
Inv
Inverse
Inverses
k  - a color
kf - a colored face
kl - a color label
Label
Len - length
List
L-  label of a face in kL, labelled permutation (rules) in LPerm.
Make
Matrix
Max
Min
Mini
Mirror
Move
Name
NB -notebook
New
Of
Off
On
Op- a symbol for an operator/operation
p - a posn (vector)
Palette
Part
Patt - a pattern (parameter)
Perm - a permuation (as rules)
Point
Poly - a polygon
Pos -positve (face labels)
Posn
Pt - a point
q - head of a colored mini face, Q is the question suffix.
r - a rotation label 
Reset
Rot - a rotation (label or matrix)
Rt   - rotation matrix at time t
Rubic
Rubik
Rule
s  - slice 
Set
Show
Sign
Signed
Slice
Split
State
Stub
T  - type head
Tag
The
Title
To
Toggle
Transformation
Triad - of faces
Tuple
Undo
Update
User
v - a vector
Val- a  value
Value
Vect - a vector
Vector
Window
Wrap
Write
$\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Identifiers", "Subsection"],

Cell["\<\
The morphemes that occur only as whole words or lexemes or in which \
the morphemes have context dependent meanings are described below.\
\>", \
"Text"],

Cell[CellGroupData[{

Cell["descriptions of small identifiers", "Subsubsection"],

Cell["(not determined by MorphemeMorphemes)", "SmallText"],

Cell["\<\
 \"a\" 
for an axis (construction of rotaxis vectors).
used transparently for a local rule (swap) variable in a small \
expression.\
\>", "Text"],

Cell["\<\
\"b\" used transparently for a local rule (swap) variable in a \
small expression.
One with no trace left:\
\>", "Text"],

Cell["\"ar\" an extra parameter for an r label substitution", "Text"],

Cell["ax,ay,az- axis labels.", "Text"],

Cell["\"bc\"=buttonContents , used as a local parameter.", "Text"],

Cell["\"bd\"=buttondata", "Text"],

Cell["\"f\" a face", "Text"],

Cell["fx,fy,fz face labels.", "Text"],

Cell["\"i\" just an index", "Text"],

Cell["\"kfrules\" color face rules", "Text"],

Cell["\"m\" amove", "Text"],

Cell["\"mirrored\" parameter-whether mirror is on.", "Text"],

Cell["\"p\" posn (vector)", "Text"],

Cell["\"p1\", \"p2\" another posn", "Text"],

Cell["\"ps\" -list of points", "Text"],

Cell["\"r\" a rot (axis) label", "Text"],

Cell["\"rx,ry,rz\" are rotation labels.", "Text"],

Cell["\"R\" a rotation matrix", "Text"],

Cell["\<\
\"r\[Sigma]\"  - face rotation permutation rules\
\>", "Text"],

Cell["\"tg\"=aTag", "Text"],

Cell["\"v\" a vector", "Text"],

Cell["\"va\" the vector of an axis", "Text"],

Cell["\<\
\"x,y,z\" component pattern
(\"x\" also used as a generic expression in a local pattern)\
\>", "Text"],

Cell["\"\[Sigma]\" a permutation", "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Function Structure", "Section"],

Cell[CellGroupData[{

Cell["comments", "Subsubsection"],

Cell[TextData[{
  "Most of the programming is standard ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " practice, with preference for rules."
}], "Text"],

Cell[TextData[{
  "Rules allow particularly transparent programming.  For example ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["list",
            FontSlant->"Italic"], 
          StyleBox["//.",
            FontSlant->"Plain"], 
          StyleBox[" ",
            FontSlant->"Plain"], 
          StyleBox[
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                "a___", ",", " ", "x_", ",", "b___", ",", "x_", ",", "c___"}],
                 "}"}], "->", 
              RowBox[{"{", 
                RowBox[{"a", ",", "x", ",", "b", ",", "c"}], "}"}]}],
            FontSlant->"Plain"]}], TraditionalForm]]],
  " is a transparent way of consolidating repititions (and preserving order). \
The fact that it is less efficient can be addressed by program transformation \
a postiori and should not compromise program style."
}], "Text"],

Cell["\<\
 We use the functional \"With\" rather than the dynamic \"Block\". \
There is seldom any loss of efficiency, but it  increases clarity.\
\>", \
"Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphics efficiency", "Subsubsection"],

Cell["\<\
The graphics for the cube had to be slowed down on some of the \
machines tried. It has just been found that on some older Macintoshes for \
instance, that they are too slow.  They could be optimized to overcome this, \
but we have not done that.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Clear", "Subsubsection"],

Cell["\<\
Experience has led to the discipline of keeping all definitions \
associated with a symbol together, and preceding them with an explicit \
\"Clear\". This is especially used with functions, except where an object \
oriented structure dictates otherwise.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Replace", "Subsubsection"],

Cell["\<\
Replace seems especially valuable. It allows us to realize \
anonymous rule based functions. It allows to simulate mutiple assignments in \
\"With\" statements. For example \"With[{{x,y}=PositionAt[t]},x-y]\" must \
descend to parts
\"With[{xy=PositionAt[t]},xy[[1]]-xy[[2]] ]\" but can be transparently \
realized as \"Replace[PositionAt[t],{x_,y_}\[Rule]x-y]\"\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Package file locations", "Section"],

Cell[TextData[{
  "The objective is to have a completely stand alone clikable so no \
alterations are made to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " directory."
}], "Text"],

Cell["\<\
The Rubic folder has subfolders with notes and the package itself. \
It should be clik startable. The start window finds the files inside the \
package directory and sets paths appropriately.\
\>", "Text"],

Cell["Here is the directory structure:", "Text"],

Cell[CellGroupData[{

Cell["RubikPkFolder:", "Text",
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{25, 0}, {0, 0}}],

Cell[":RubikCheckStart.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikCheckStart.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikGraphicsFns.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikGraphicsFns.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikGraphicsPk.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikGraphicsPk.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":Rubik Instructions.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikInterfaceFns.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikInterfaceFns.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikInterfacePk.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikInterfacePk.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikMakeWindowFns.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikMakeWindowFns.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikMakeWindowPk.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikMakeWindowPk.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":Rubik Planar Group Doc.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikUserButtonsFns.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikUserButtonsFns.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikUserButtonsPk.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":RubikUserButtonsPk.nb", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[":ToolsCheckPk.m", "Text",
  CellMargins->{{30, 0}, {0, 0}}],

Cell[CellGroupData[{

Cell[":ToolsFolder[\[Ellipsis]]", "Text",
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{45, 0}, {0, 0}}],

Cell[CellGroupData[{

Cell[":GraphicsFolder[\[Ellipsis]]", "Text",
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{55, 0}, {0, 0}}],

Cell[":DirectDisplayFns.m", "Text",
  CellMargins->{{50, 0}, {0, 0}}],

Cell[":DirectDisplayPk.m", "Text",
  CellMargins->{{50, 0}, {0, 0}}],

Cell[":Rofa.m", "Text",
  CellMargins->{{50, 0}, {0, 0}}]
}, Closed]],

Cell[":UtilitiesFolder[:WarningWindowFns.m, :WarningWindowPk.m]", "Text",
  CellMargins->{{40, 0}, {0, 0}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Loading the Package", "Section"],

Cell[CellGroupData[{

Cell["initialization", "Subsubsection"],

Cell["\<\
The \"Start Rubik Window\" sets \"ToolsPathSep\" and \
\"RubikPkPathSep\" and calls the generic tools checking package \
\"ToolsCheckPk.m\" and initializes it with the ToolsPathSep which is added to \
the system path.\
\>", "Text"],

Cell["\<\
Only a frontend notebook can determine where it is, so the Start \
window also sets the global RubikPkPathSep. This is used by its own buttons \
and also as a parameter to the \"RubicCheckStart\" that checks the Rubik \
files are in place and loads the package.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Comments", "Subsubsection"],

Cell[TextData[{
  "This completely self contained arrangement of files allows the package to \
be \n\"clik started\" from anywhere-on unix machines where access to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " AddOns is restricted, off servers etc."
}], "Text"],

Cell["\<\
The \"Tools\" folder holds generic tools, but is independent of the \
fixed \"AddOns\" file structure.\
\>", "Text"],

Cell["\<\
The initialization for this arrangement of files does use a few \
global parameters for the first initialization paths and checks.\
\>", "Text"],

Cell["\<\
It also \"captures\" the global tools path. This would only have an \
effect during 
development when another tools directory was in use.\
\>", "Text"],

Cell[TextData[{
  "When a user is already running ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "  (even just restarting the package) the system path may have been altered \
so the tools are coming from another folder."
}], "Text"],

Cell["\<\
If Mathematica is being run with a remote kernel (e.g. a cluster of \
machines running front ends in a teaching laboratory) it is difficult to \
allow a clickable notebook launch, as the kernel would need to read files \
thru the frontend.\
\>", "Text"],

Cell["\<\
If a file is \"Get\"'d, it can determine its directory from $Input, \
but if it is in a nested \"Get\" this may only return a relative directory. A \
package seems unable to determine from where it is being read. Thus it is \
only the intial notebook that can set the directory paths.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell["Rubik Package", "Subtitle"],

Cell["\<\
See the separate \"Rubik Geometry\" notes for the mathematical \
framework.\
\>", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Description of RubikGraphics", "Section"],

Cell[CellGroupData[{

Cell["Axes,rotations, positions and faces", "Subsubsection"],

Cell["\<\
Define the constant symbols used for axis and their indexing and \
vectors.\
\>", "Text"],

Cell[TextData[{
  "Define the constant symbols, ",
  Cell[BoxData[
      FormBox["AllRot90Labels", TraditionalForm]]],
  ", used for rotation axis and their indexing and vectors.\nDefine the \
permutation action of ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[ScriptCapitalR]", "3"], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox["AllRot90sToAxisPermRules", TraditionalForm]]],
  ", and their mapping to matrices ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", "All", ")"}], "Rot90ToMatrixRules"}], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "Define the slice domain ",
  Cell[BoxData[
      FormBox["RubikPosnDom", TraditionalForm]]],
  ", and the set of positions Posns:"
}], "Text"],

Cell[TextData[{
  "Define the (positive) face labels ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", "Pos", ")"}], "FaceLabels"}], TraditionalForm]]],
  ". Define the effect of the rotations on the faces ",
  Cell[BoxData[
      FormBox["AllRot90sToFacePermRules", TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "Define the color labels ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"kL", "[", "i", "]"}], ",", " ", 
          RowBox[{"i", "=", "1"}], ",", "\[Ellipsis]", ",", "6"}], 
        TraditionalForm]]],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphics for faces", "Subsubsection"],

Cell[TextData[{
  "Define the map from faces to their polygons ",
  Cell[BoxData[
      FormBox["FaceLabelToPolyRules", TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "Choose the set of colors to use: ",
  Cell[BoxData[
      FormBox["FaceColors", TraditionalForm]]],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Configurations", "Subsubsection"],

Cell[TextData[{
  "Define the faces associated with each position: ",
  Cell[BoxData[
      FormBox["FaceLabelsOfPosn", TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "Construct the ",
  Cell[BoxData[
      FormBox["InitialColoredMiniFaceConfiguration", TraditionalForm]]],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Rubik Group", "Subsubsection"],

Cell[TextData[{
  "Define the inverse of a generator ",
  Cell[BoxData[
      FormBox["InvOfRubikGen", TraditionalForm]]],
  ", the patterns for selecting the slice domain of a generator, and axis of \
generators. "
}], "Text"],

Cell[TextData[{
  "The important function ",
  Cell[BoxData[
      FormBox["qRulesOfRubikGen", TraditionalForm]]],
  " constructs the colored mini face rules that implement the action of a \
Rubik generator."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphics", "Subsubsection"],

Cell[TextData[{
  "Use ",
  Cell[BoxData[
      FormBox["qToGraphicsRule", TraditionalForm]]],
  " to map colored mini faces to graphics elements."
}], "Text"],

Cell["\<\
Construct mirrors by slicing faces off the graphics of a \
cube.\
\>", "Text"],

Cell[TextData[{
  "The actual construction of the graphics and display of a cube (colored \
mini face configuration) is done with ",
  Cell[BoxData[
      FormBox[
        RowBox[{"DisplayCube", "[", 
          RowBox[{"aCube_", ",", "anNB_", ",", "tg", ",", "Mirrored_"}], 
          "]"}], TraditionalForm]]],
  ". "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Animation", "Subsubsection"],

Cell["\<\
For animation, separate off the slice that is to rotate (and add \
interior faces) and then animate.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Interface Functions", "Section"],

Cell[CellGroupData[{

Cell["state", "Subsubsection"],

Cell["\<\
As discussed in Rubik geometry notes, all the configurations of the \
cube are given by sets of colored mini-faces. The interface maintains a \
currentlabelcubefaces.\
\>", "Text"],

Cell["\<\
The cube interface has state variables:
animation:animationon
mirror: showmirror,
inverse buttons:inverseson\
\>", "Text"],

Cell["\<\
The basic/simplest operation is achieved by a permutation rule of \
the colored minifaces.
The built in generators (or their inverses) can be animated.\
\>", "Text"],

Cell["\<\
Each button operation 
-performs the (inverted) operation on the cube (via animation if \
appropriate).\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Current moves and Undo", "Subsubsection"],

Cell["\<\
The current moves must be able to display formulae for the action.
The undo must be able to find the operation associated with a formula and to \
compute its inverse.\
\>", "Text"],

Cell["\<\
Each button operation sends an object with: 
-a formula for the operation
-a method that returns the colored miniface permutatation rules for the \
inverse operation,
(both incorporating the inverse when it is on).\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["User Buttons", "Section"],

Cell["\<\
For the user defined buttons, maintain a list of the (mutually) \
defined actions,
each consisting of its formula (in terms of previously defined buttons), a \
label for the operation, and a permutation of the miniface indices.\
\>", \
"Text"],

Cell["\<\
These button functions must map the miniface index permutation to \
permutation rules for the colored minifaces, and provide a method for the \
inverse. Both these must take account of the current inverse state.\
\>", \
"Text"],

Cell["\<\
When the user  defines a button, it is created and appended before \
the \"afteralllists\" tagged cell.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Make Rubik's Window", "Section"],

Cell["\<\
These functions construct the buttons, cells and notebook for the \
interactive Rubik's Window.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Extensions", "Section"],

Cell[TextData[{
  "This is the basic package. A user may save the UserMovesList at any time \
(with a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " command). There is a function for loading such a saved file and making it \
the current user button window. Another configuration can be loaded for \
\"solving\"."
}], "Text"],

Cell["\<\
Other versions allow, generating random initial configurations,  \
animation of sequences of moves, automatic solution, automate handling user \
button documents, display different \"filtered\" graphics.\
\>", "Text"]
}, Closed]]
},
FrontEndVersion->"4.0 for Macintosh",
ScreenRectangle->{{0, 832}, {0, 604}},
CellGrouping->Manual,
WindowSize->{596, 575},
WindowMargins->{{35, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
MacintoshSystemPageSetup->"\<\
00`0004/0B`0000036`9H?o4okD<Z0V[:P85:0?l0@00004/0B`0000036`9H001
1@00I0000@400@0000000BL?004000;^L>000000000000000000000000000?o4
okD<Z0V[00P000000040000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 43, 0, 98, "Title"],
Cell[1763, 51, 27, 0, 62, "Subtitle"],

Cell[CellGroupData[{
Cell[1815, 55, 45, 0, 50, "Section"],
Cell[1863, 57, 298, 5, 62, "Text"],
Cell[2164, 64, 229, 4, 46, "Text"],
Cell[2396, 70, 238, 5, 46, "Text"],
Cell[2637, 77, 203, 4, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[2877, 86, 33, 0, 30, "Section"],
Cell[2913, 88, 574, 9, 94, "Text"],
Cell[3490, 99, 197, 4, 62, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3724, 108, 30, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[3779, 112, 45, 0, 42, "Subsubsection"],
Cell[3827, 114, 63, 0, 30, "Text"],
Cell[3893, 116, 380, 8, 110, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4310, 129, 46, 0, 28, "Subsubsection"],
Cell[4359, 131, 240, 5, 46, "Text"],
Cell[4602, 138, 110, 3, 30, "Text"],
Cell[4715, 143, 305, 5, 62, "Text"],
Cell[5023, 150, 148, 3, 46, "Text"],
Cell[5174, 155, 758, 17, 98, "Text"],
Cell[5935, 174, 221, 5, 62, "Text"],
Cell[6159, 181, 117, 3, 30, "Text"],
Cell[6279, 186, 352, 10, 64, "Text"],
Cell[6634, 198, 157, 3, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6828, 206, 31, 0, 30, "Subsection"],

Cell[CellGroupData[{
Cell[6884, 210, 33, 0, 42, "Subsubsection"],
Cell[6920, 212, 166, 4, 46, "Text"],
Cell[7089, 218, 122, 3, 30, "Text"],
Cell[7214, 223, 183, 4, 46, "Text"],
Cell[7400, 229, 248, 5, 46, "Text"],
Cell[7651, 236, 112, 3, 30, "Text"],
Cell[7766, 241, 134, 3, 26, "SmallText"]
}, Closed]],

Cell[CellGroupData[{
Cell[7937, 249, 57, 0, 28, "Subsubsection"],
Cell[7997, 251, 109, 3, 30, "Text"],
Cell[8109, 256, 50, 0, 30, "Text"],
Cell[8162, 258, 1700, 137, 2158, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[9911, 401, 33, 0, 30, "Subsection"],
Cell[9947, 403, 162, 4, 46, "Text"],

Cell[CellGroupData[{
Cell[10134, 411, 58, 0, 42, "Subsubsection"],
Cell[10195, 413, 58, 0, 26, "SmallText"],
Cell[10256, 415, 155, 5, 62, "Text"],
Cell[10414, 422, 130, 4, 46, "Text"],
Cell[10547, 428, 69, 0, 30, "Text"],
Cell[10619, 430, 38, 0, 30, "Text"],
Cell[10660, 432, 66, 0, 30, "Text"],
Cell[10729, 434, 33, 0, 30, "Text"],
Cell[10765, 436, 28, 0, 30, "Text"],
Cell[10796, 438, 37, 0, 30, "Text"],
Cell[10836, 440, 35, 0, 30, "Text"],
Cell[10874, 442, 44, 0, 30, "Text"],
Cell[10921, 444, 27, 0, 30, "Text"],
Cell[10951, 446, 60, 0, 30, "Text"],
Cell[11014, 448, 35, 0, 30, "Text"],
Cell[11052, 450, 43, 0, 30, "Text"],
Cell[11098, 452, 38, 0, 30, "Text"],
Cell[11139, 454, 40, 0, 30, "Text"],
Cell[11182, 456, 49, 0, 30, "Text"],
Cell[11234, 458, 39, 0, 30, "Text"],
Cell[11276, 460, 72, 2, 30, "Text"],
Cell[11351, 464, 27, 0, 30, "Text"],
Cell[11381, 466, 30, 0, 30, "Text"],
Cell[11414, 468, 44, 0, 30, "Text"],
Cell[11461, 470, 112, 3, 46, "Text"],
Cell[11576, 475, 42, 0, 30, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[11679, 482, 37, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[11741, 486, 33, 0, 42, "Subsubsection"],
Cell[11777, 488, 163, 5, 32, "Text"],
Cell[11943, 495, 894, 23, 78, "Text"],
Cell[12840, 520, 161, 4, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13038, 529, 44, 0, 28, "Subsubsection"],
Cell[13085, 531, 270, 5, 62, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13392, 541, 30, 0, 28, "Subsubsection"],
Cell[13425, 543, 277, 5, 62, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13739, 553, 32, 0, 28, "Subsubsection"],
Cell[13774, 555, 386, 7, 94, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[14209, 568, 41, 0, 30, "Section"],
Cell[14253, 570, 196, 6, 32, "Text"],
Cell[14452, 578, 215, 4, 46, "Text"],
Cell[14670, 584, 48, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[14743, 588, 96, 2, 16, "Text"],
Cell[14842, 592, 68, 1, 16, "Text"],
Cell[14913, 595, 69, 1, 16, "Text"],
Cell[14985, 598, 69, 1, 16, "Text"],
Cell[15057, 601, 70, 1, 16, "Text"],
Cell[15130, 604, 68, 1, 16, "Text"],
Cell[15201, 607, 69, 1, 16, "Text"],
Cell[15273, 610, 72, 1, 16, "Text"],
Cell[15348, 613, 70, 1, 16, "Text"],
Cell[15421, 616, 71, 1, 16, "Text"],
Cell[15495, 619, 69, 1, 16, "Text"],
Cell[15567, 622, 70, 1, 16, "Text"],
Cell[15640, 625, 71, 1, 16, "Text"],
Cell[15714, 628, 72, 1, 16, "Text"],
Cell[15789, 631, 70, 1, 16, "Text"],
Cell[15862, 634, 71, 1, 16, "Text"],
Cell[15936, 637, 76, 1, 16, "Text"],
Cell[16015, 640, 72, 1, 16, "Text"],
Cell[16090, 643, 73, 1, 16, "Text"],
Cell[16166, 646, 71, 1, 16, "Text"],
Cell[16240, 649, 72, 1, 16, "Text"],
Cell[16315, 652, 65, 1, 16, "Text"],

Cell[CellGroupData[{
Cell[16405, 657, 107, 2, 16, "Text"],

Cell[CellGroupData[{
Cell[16537, 663, 110, 2, 16, "Text"],
Cell[16650, 667, 69, 1, 16, "Text"],
Cell[16722, 670, 68, 1, 16, "Text"],
Cell[16793, 673, 57, 1, 16, "Text"]
}, Closed]],
Cell[16865, 677, 107, 1, 20, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[17033, 685, 38, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[17096, 689, 39, 0, 42, "Subsubsection"],
Cell[17138, 691, 241, 5, 46, "Text"],
Cell[17382, 698, 285, 5, 62, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[17704, 708, 33, 0, 28, "Subsubsection"],
Cell[17740, 710, 278, 6, 64, "Text"],
Cell[18021, 718, 126, 3, 30, "Text"],
Cell[18150, 723, 154, 3, 46, "Text"],
Cell[18307, 728, 161, 4, 46, "Text"],
Cell[18471, 734, 242, 6, 48, "Text"],
Cell[18716, 742, 263, 5, 62, "Text"],
Cell[18982, 749, 308, 5, 62, "Text"]
}, Closed]]
}, Closed]],
Cell[19317, 758, 33, 0, 51, "Subtitle"],
Cell[19353, 760, 106, 3, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[19484, 767, 47, 0, 50, "Section"],

Cell[CellGroupData[{
Cell[19556, 771, 60, 0, 42, "Subsubsection"],
Cell[19619, 773, 99, 3, 30, "Text"],
Cell[19721, 778, 595, 19, 62, "Text"],
Cell[20319, 799, 161, 5, 30, "Text"],
Cell[20483, 806, 328, 10, 46, "Text"],
Cell[20814, 818, 259, 9, 30, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[21110, 832, 43, 0, 28, "Subsubsection"],
Cell[21156, 834, 157, 5, 30, "Text"],
Cell[21316, 841, 136, 5, 30, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[21489, 851, 39, 0, 28, "Subsubsection"],
Cell[21531, 853, 157, 5, 30, "Text"],
Cell[21691, 860, 142, 5, 30, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[21870, 870, 36, 0, 28, "Subsubsection"],
Cell[21909, 872, 227, 6, 46, "Text"],
Cell[22139, 880, 220, 6, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[22396, 891, 33, 0, 28, "Subsubsection"],
Cell[22432, 893, 159, 5, 30, "Text"],
Cell[22594, 900, 88, 3, 30, "Text"],
Cell[22685, 905, 330, 9, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[23052, 919, 34, 0, 28, "Subsubsection"],
Cell[23089, 921, 124, 3, 30, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[23262, 930, 38, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[23325, 934, 30, 0, 42, "Subsubsection"],
Cell[23358, 936, 190, 4, 46, "Text"],
Cell[23551, 942, 132, 5, 78, "Text"],
Cell[23686, 949, 175, 4, 46, "Text"],
Cell[23864, 955, 127, 4, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[24028, 964, 47, 0, 28, "Subsubsection"],
Cell[24078, 966, 190, 4, 46, "Text"],
Cell[24271, 972, 238, 6, 78, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[24558, 984, 31, 0, 30, "Section"],
Cell[24592, 986, 253, 6, 62, "Text"],
Cell[24848, 994, 237, 5, 46, "Text"],
Cell[25088, 1001, 127, 3, 30, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[25252, 1009, 38, 0, 30, "Section"],
Cell[25293, 1011, 119, 3, 30, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[25449, 1019, 29, 0, 30, "Section"],
Cell[25481, 1021, 336, 8, 64, "Text"],
Cell[25820, 1031, 227, 4, 46, "Text"]
}, Closed]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

